@@ -44,21 +44,19 @@ from ryu.services.protocols.bgp.rtconf.common import REFRESH_MAX_EOR_TIME
 from ryu.services.protocols.bgp.rtconf.common import REFRESH_STALEPATH_TIME
 from ryu.services.protocols.bgp.rtconf.common import ROUTER_ID
 from ryu.services.protocols.bgp.rtconf import neighbors
+from ryu.services.protocols.bgp.rtconf import vrfs
 from ryu.services.protocols.bgp.utils.dictconfig import dictConfig
 from ryu.services.protocols.bgp.utils.validation import is_valid_ipv4
 
 LOG = logging.getLogger('bgpspeaker.application')
 CONF = cfg.CONF
 
-DEFAULT_CONFIG_PATH = path.dirname(path.abspath(__file__)) + \
-    '/bgp_sample_conf.py'
-
 CONF.register_opts([
     cfg.IntOpt('bind-port', default=50002, help='rpc-port'),
     cfg.StrOpt('bind-ip', default='0.0.0.0', help='rpc-bind-ip'),
-    cfg.StrOpt('bgp-config-file', default=DEFAULT_CONFIG_PATH,
+    cfg.StrOpt('bgp-config-file', default=None,
                help='bgp-config-file')
-    ])
+])
 
 
 @add_bgp_error_metadata(code=BIN_ERROR,
@@ -102,7 +100,7 @@ class BGPSpeaker(RyuApp):
 
         super(BGPSpeaker, self).start()
 
-        t.wait()
+        return t
 
     @classmethod
     def validate_rpc_ip(cls, ip):
@@ -206,8 +204,9 @@ class BGPSpeaker(RyuApp):
         All valid VRFs are loaded.
         """
         vpns_conf = routing_settings.setdefault('vpns', {})
-        for vrf in vpns_conf:
+        for vrfname, vrf in vpns_conf.iteritems():
             try:
+                vrf[vrfs.VRF_NAME] = vrfname
                 call('vrf.create', **vrf)
                 LOG.debug('Added vrf  %s' % str(vrf))
             except RuntimeConfigError as e:
