@@ -352,9 +352,9 @@ class CoreService(Factory, Activity):
     def unregister_flexinet_sink(self, sink):
         self._sinks.remove(sink)
 
-    def update_flexinet_peers(self, path, route_disc):
+    def update_flexinet_peers(self, path, route_dist):
         for sink in self._sinks:
-            out_route = FlexinetOutgoingRoute(path, route_disc)
+            out_route = FlexinetOutgoingRoute(path, route_dist)
             sink.enque_outgoing_msg(out_route)
 
     def on_peer_added(self, peer):
@@ -424,4 +424,7 @@ class CoreService(Factory, Activity):
             subcode = BGP_ERROR_SUB_CONNECTION_COLLISION_RESOLUTION
             bgp_proto.send_notification(code, subcode)
         else:
+            bind_ip, bind_port = socket.getsockname()
+            peer._host_bind_ip = bind_ip
+            peer._host_bind_port = bind_port
             self._spawn_activity(bgp_proto, peer)
