@@ -20,7 +20,6 @@ from ryu.lib.packet.bgp import RF_IPv6_UC
 from ryu.lib.packet.bgp import RF_IPv4_VPN
 from ryu.lib.packet.bgp import RF_IPv6_VPN
 from ryu.lib.packet.bgp import RF_RTC_UC
-from ryu.lib.packet.bgp import BGPPathAttributeNextHop
 from ryu.lib.packet.bgp import BGPPathAttributeOrigin
 from ryu.lib.packet.bgp import BGPPathAttributeAsPath
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_ORIGIN
@@ -432,10 +431,10 @@ class TableCoreManager(object):
         if interested_tables:
             # We iterate over all VRF tables that are interested in the RT
             # of the given path and import this path into them.
-            route_disc = vpn_path.nlri.route_disc
+            route_dist = vpn_path.nlri.route_dist
             for vrf_table in interested_tables:
                 if not (vpn_path.source is None
-                        and route_disc == vrf_table.vrf_conf.route_dist):
+                        and route_dist == vrf_table.vrf_conf.route_dist):
                     update_vrf_dest = vrf_table.import_vpn_path(vpn_path)
                     # Queue the destination for further processing.
                     if update_vrf_dest is not None:
@@ -489,7 +488,7 @@ class TableCoreManager(object):
         src_ver_num = 1
         peer = None
         # set mandatory path attributes
-        nexthop = BGPPathAttributeNextHop("0.0.0.0")
+        nexthop = '0.0.0.0'
         origin = BGPPathAttributeOrigin(BGP_ATTR_ORIGIN_IGP)
         aspath = BGPPathAttributeAsPath([[]])
 
