@@ -161,7 +161,7 @@ BPR_UNKNOWN = 'Unknown'
 BPR_ONLY_PATH = 'Only Path'
 BPR_REACHABLE_NEXT_HOP = 'Reachable Next Hop'
 BPR_HIGHEST_WEIGHT = 'Highest Weight'
-BPR_LOCAL_PREF = 'Local Pref.'
+BPR_LOCAL_PREF = 'Local Pref'
 BPR_LOCAL_ORIGIN = 'Local Origin'
 BPR_ASPATH = 'AS Path'
 BPR_ORIGIN = 'Origin'
