@@ -38,6 +38,8 @@ from ryu.lib.packet.bgp import BGP_MSG_ROUTE_REFRESH
 from ryu.lib.packet.bgp import BGP_CAP_ENHANCED_ROUTE_REFRESH
 from ryu.lib.packet.bgp import BGP_CAP_MULTIPROTOCOL
 from ryu.lib.packet.bgp import BGP_CAP_ROUTE_REFRESH
+from ryu.lib.packet.bgp import BGP_ERROR_HOLD_TIMER_EXPIRED
+from ryu.lib.packet.bgp import BGP_ERROR_SUB_HOLD_TIMER_EXPIRED
 
 from ryu.services.protocols.bgp.base import Activity
 from ryu.services.protocols.bgp.base import add_bgp_error_metadata
@@ -371,15 +373,18 @@ class BgpProtocol(Protocol, Activity):
             raise BgpProtocolException('Tried to send message to peer when '
                                        'this protocol instance is not started'
                                        ' or is no longer is started state.')
+        # get peername before senging msg because sending msg can occur
+        # conncetion lost
+        peername = self.get_peername()
         self._socket.sendall(msg.serialize())
+
         if msg.type == BGP_MSG_NOTIFICATION:
             LOG.error('Sent notification to %s >> %s' %
-                      (self.get_peername(), msg))
+                      (peername, msg))
 
             self._signal_bus.bgp_notification_sent(self._peer, msg)
-
         else:
-            LOG.debug('Sent msg to %s >> %s' % (self.get_peername(), msg))
+            LOG.debug('Sent msg to %s >> %s' % (peername, msg))
 
     def stop(self):
         Activity.stop(self)
