@@ -18,11 +18,13 @@ import inspect
 import json
 import logging
 import math
+import netaddr
 import os
 import signal
 import sys
 import time
 import traceback
+from random import randint
 
 from ryu import cfg
 
@@ -51,15 +53,15 @@ from ryu.ofproto import ofproto_v1_3_parser
 
 """ Required test network:
 
-                      +-----------+
-           +----------| target sw | The switch to be tested
-           |          +-----------+
-    +------------+      (1)   (2)
-    | controller |       |     |
-    +------------+      (1)   (2)
-           |          +-----------+
-           +----------| tester sw | OpenFlow Switch
-                      +-----------+
+                      +-------------------+
+           +----------|     target sw     | The switch to be tested
+           |          +-------------------+
+    +------------+      (1)   (2)   (3)
+    | controller |       |     |     |
+    +------------+      (1)   (2)   (3)
+           |          +-------------------+
+           +----------|     tester sw     | OpenFlow Switch
+                      +-------------------+
 
       (X) : port number
 
@@ -86,7 +88,7 @@ INTERVAL = 1  # sec
 WAIT_TIMER = 3  # sec
 CONTINUOUS_THREAD_INTVL = float(0.01)  # sec
 CONTINUOUS_PROGRESS_SPAN = 3  # sec
-THROUGHPUT_PRIORITY = ofproto_v1_3.OFP_DEFAULT_PRIORITY+1
+THROUGHPUT_PRIORITY = ofproto_v1_3.OFP_DEFAULT_PRIORITY + 1
 THROUGHPUT_COOKIE = THROUGHPUT_PRIORITY
 THROUGHPUT_THRESHOLD = float(0.10)  # expected throughput plus/minus 10 %
 
@@ -99,6 +101,7 @@ KEY_DESC = 'description'
 KEY_PREREQ = 'prerequisite'
 KEY_FLOW = 'OFPFlowMod'
 KEY_METER = 'OFPMeterMod'
+KEY_GROUP = 'OFPGroupMod'
 KEY_TESTS = 'tests'
 KEY_INGRESS = 'ingress'
 KEY_EGRESS = 'egress'
@@ -131,6 +134,9 @@ STATE_THROUGHPUT_FLOW_INSTALL = 14
 STATE_THROUGHPUT_FLOW_EXIST_CHK = 15
 STATE_GET_THROUGHPUT = 16
 STATE_THROUGHPUT_CHK = 17
+STATE_INIT_GROUP = 18
+STATE_GROUP_INSTALL = 19
+STATE_GROUP_EXIST_CHK = 20
 
 STATE_DISCONNECTED = 99
 
@@ -165,6 +171,9 @@ MSG = {STATE_INIT_FLOW:
        STATE_METER_INSTALL:
        {TIMEOUT: 'Failed to add meters: barrier request timeout.',
         RCV_ERR: 'Failed to add meters: %(err_msg)s'},
+       STATE_GROUP_INSTALL:
+       {TIMEOUT: 'Failed to add groups: barrier request timeout.',
+        RCV_ERR: 'Failed to add groups: %(err_msg)s'},
        STATE_FLOW_EXIST_CHK:
        {FAILURE: 'Added incorrect flows: %(flows)s',
         TIMEOUT: 'Failed to add flows: flow stats request timeout.',
@@ -173,6 +182,10 @@ MSG = {STATE_INIT_FLOW:
        {FAILURE: 'Added incorrect meters: %(meters)s',
         TIMEOUT: 'Failed to add meters: meter config stats request timeout.',
         RCV_ERR: 'Failed to add meters: %(err_msg)s'},
+       STATE_GROUP_EXIST_CHK:
+       {FAILURE: 'Added incorrect groups: %(groups)s',
+        TIMEOUT: 'Failed to add groups: group desc stats request timeout.',
+        RCV_ERR: 'Failed to add groups: %(err_msg)s'},
        STATE_TARGET_PKT_COUNT:
        {TIMEOUT: 'Failed to request port stats from target: request timeout.',
         RCV_ERR: 'Failed to request port stats from target: %(err_msg)s'},
@@ -257,8 +270,8 @@ class OfTester(app_manager.RyuApp):
         test_dir = CONF['test-switch']['dir']
         self.logger.info('Test files directory = %s', test_dir)
 
-        self.target_sw = TargetSw(DummyDatapath(), self.logger)
-        self.tester_sw = TesterSw(DummyDatapath(), self.logger)
+        self.target_sw = OpenFlowSw(DummyDatapath(), self.logger)
+        self.tester_sw = OpenFlowSw(DummyDatapath(), self.logger)
         self.state = STATE_INIT_FLOW
         self.sw_waiter = None
         self.waiter = None
@@ -381,16 +394,24 @@ class OfTester(app_manager.RyuApp):
         try:
             # Initialize.
             self._test(STATE_INIT_METER)
-            self._test(STATE_INIT_FLOW)
-            self._test(STATE_INIT_THROUGHPUT_FLOW)
+            self._test(STATE_INIT_GROUP)
+            self._test(STATE_INIT_FLOW, self.target_sw)
+            self._test(STATE_INIT_THROUGHPUT_FLOW, self.tester_sw,
+                       THROUGHPUT_COOKIE)
             # Install flows.
             for flow in test.prerequisite:
                 if isinstance(flow, ofproto_v1_3_parser.OFPFlowMod):
                     self._test(STATE_FLOW_INSTALL, self.target_sw, flow)
-                    self._test(STATE_FLOW_EXIST_CHK, self.target_sw, flow)
+                    self._test(STATE_FLOW_EXIST_CHK,
+                               self.target_sw.send_flow_stats, flow)
                 elif isinstance(flow, ofproto_v1_3_parser.OFPMeterMod):
-                    self._test(STATE_METER_INSTALL, flow)
-                    self._test(STATE_METER_EXIST_CHK, flow)
+                    self._test(STATE_METER_INSTALL, self.target_sw, flow)
+                    self._test(STATE_METER_EXIST_CHK,
+                               self.target_sw.send_meter_config_stats, flow)
+                elif isinstance(flow, ofproto_v1_3_parser.OFPGroupMod):
+                    self._test(STATE_GROUP_INSTALL, self.target_sw, flow)
+                    self._test(STATE_GROUP_EXIST_CHK,
+                               self.target_sw.send_group_desc_stats, flow)
             # Do tests.
             for pkt in test.tests:
 
@@ -407,7 +428,7 @@ class OfTester(app_manager.RyuApp):
                         self._test(STATE_THROUGHPUT_FLOW_INSTALL,
                                    self.tester_sw, flow)
                         self._test(STATE_THROUGHPUT_FLOW_EXIST_CHK,
-                                   self.tester_sw, flow)
+                                   self.tester_sw.send_flow_stats, flow)
                     start = self._test(STATE_GET_THROUGHPUT)
                 elif KEY_TBL_MISS in pkt:
                     before_stats = self._test(STATE_GET_MATCH_COUNT)
@@ -476,29 +497,32 @@ class OfTester(app_manager.RyuApp):
 
     def _output_test_report(self, report):
         self.logger.info('%s--- Test report ---', os.linesep)
-        ok_count = error_count = 0
+        error_count = 0
         for result_type in sorted(report.keys()):
             test_descriptions = report[result_type]
             if result_type == TEST_OK:
-                ok_count = len(test_descriptions)
                 continue
             error_count += len(test_descriptions)
             self.logger.info('%s(%d)', result_type, len(test_descriptions))
             for file_desc, test_desc in test_descriptions:
                 self.logger.info('    %-40s %s', file_desc, test_desc)
         self.logger.info('%s%s(%d) / %s(%d)', os.linesep,
-                         TEST_OK, ok_count, TEST_ERROR, error_count)
+                         TEST_OK, len(report.get(TEST_OK, [])),
+                         TEST_ERROR, error_count)
 
     def _test(self, state, *args):
         test = {STATE_INIT_FLOW: self._test_initialize_flow,
-                STATE_INIT_THROUGHPUT_FLOW: self._test_initialize_flow_tester,
-                STATE_INIT_METER: self._test_initialize_meter,
-                STATE_FLOW_INSTALL: self._test_flow_install,
-                STATE_THROUGHPUT_FLOW_INSTALL: self._test_flow_install,
-                STATE_METER_INSTALL: self._test_meter_install,
-                STATE_FLOW_EXIST_CHK: self._test_flow_exist_check,
-                STATE_THROUGHPUT_FLOW_EXIST_CHK: self._test_flow_exist_check,
-                STATE_METER_EXIST_CHK: self._test_meter_exist_check,
+                STATE_INIT_THROUGHPUT_FLOW: self._test_initialize_flow,
+                STATE_INIT_METER: self.target_sw.del_meters,
+                STATE_INIT_GROUP: self.target_sw.del_groups,
+                STATE_FLOW_INSTALL: self._test_msg_install,
+                STATE_THROUGHPUT_FLOW_INSTALL: self._test_msg_install,
+                STATE_METER_INSTALL: self._test_msg_install,
+                STATE_GROUP_INSTALL: self._test_msg_install,
+                STATE_FLOW_EXIST_CHK: self._test_exist_check,
+                STATE_THROUGHPUT_FLOW_EXIST_CHK: self._test_exist_check,
+                STATE_METER_EXIST_CHK: self._test_exist_check,
+                STATE_GROUP_EXIST_CHK: self._test_exist_check,
                 STATE_TARGET_PKT_COUNT: self._test_get_packet_count,
                 STATE_TESTER_PKT_COUNT: self._test_get_packet_count,
                 STATE_FLOW_MATCH_CHK: self._test_flow_matching_check,
@@ -515,35 +539,8 @@ class OfTester(app_manager.RyuApp):
         self.state = state
         return test[state](*args)
 
-    def _test_initialize_flow(self):
-        xid = self.target_sw.del_test_flow()
-        self.send_msg_xids.append(xid)
-
-        xid = self.target_sw.send_barrier_request()
-        self.send_msg_xids.append(xid)
-
-        self._wait()
-        assert len(self.rcv_msgs) == 1
-        msg = self.rcv_msgs[0]
-        assert isinstance(msg, ofproto_v1_3_parser.OFPBarrierReply)
-
-    def _test_initialize_flow_tester(self):
-        xid = self.tester_sw.del_flows_for_throughput_analysis()
-        self.send_msg_xids.append(xid)
-
-        xid = self.tester_sw.send_barrier_request()
-        self.send_msg_xids.append(xid)
-
-        self._wait()
-        assert len(self.rcv_msgs) == 1
-        msg = self.rcv_msgs[0]
-        assert isinstance(msg, ofproto_v1_3_parser.OFPBarrierReply)
-
-    def _test_initialize_meter(self):
-        self.target_sw.del_test_meter()
-
-    def _test_flow_install(self, datapath, flow):
-        xid = datapath.add_flow(flow_mod=flow)
+    def _test_initialize_flow(self, datapath, cookie=0):
+        xid = datapath.del_flows(cookie)
         self.send_msg_xids.append(xid)
 
         xid = datapath.send_barrier_request()
@@ -554,11 +551,11 @@ class OfTester(app_manager.RyuApp):
         msg = self.rcv_msgs[0]
         assert isinstance(msg, ofproto_v1_3_parser.OFPBarrierReply)
 
-    def _test_meter_install(self, meter):
-        xid = self.target_sw._send_msg(meter)
+    def _test_msg_install(self, datapath, message):
+        xid = datapath.send_msg(message)
         self.send_msg_xids.append(xid)
 
-        xid = self.target_sw.send_barrier_request()
+        xid = datapath.send_barrier_request()
         self.send_msg_xids.append(xid)
 
         self._wait()
@@ -566,38 +563,45 @@ class OfTester(app_manager.RyuApp):
         msg = self.rcv_msgs[0]
         assert isinstance(msg, ofproto_v1_3_parser.OFPBarrierReply)
 
-    def _test_flow_exist_check(self, datapath, flow_mod):
-        xid = datapath.send_flow_stats()
+    def _test_exist_check(self, method, message):
+        method_dict = {
+            OpenFlowSw.send_flow_stats.__name__: {
+                'reply': ofproto_v1_3_parser.OFPFlowStatsReply,
+                'compare': self._compare_flow
+            },
+            OpenFlowSw.send_meter_config_stats.__name__: {
+                'reply': ofproto_v1_3_parser.OFPMeterConfigStatsReply,
+                'compare': self._compare_meter
+            },
+            OpenFlowSw.send_group_desc_stats.__name__: {
+                'reply': ofproto_v1_3_parser.OFPGroupDescStatsReply,
+                'compare': self._compare_group
+            }
+        }
+        xid = method()
         self.send_msg_xids.append(xid)
         self._wait()
 
         ng_stats = []
         for msg in self.rcv_msgs:
-            assert isinstance(msg, ofproto_v1_3_parser.OFPFlowStatsReply)
+            assert isinstance(msg, method_dict[method.__name__]['reply'])
             for stats in msg.body:
-                result, stats = self._compare_flow(stats, flow_mod)
+                result, stats = method_dict[method.__name__]['compare'](
+                    stats, message)
                 if result:
                     return
                 else:
                     ng_stats.append(stats)
-        raise TestFailure(self.state, flows=', '.join(ng_stats))
 
-    def _test_meter_exist_check(self, meter_mod):
-        xid = self.target_sw.send_meter_config_stats()
-        self.send_msg_xids.append(xid)
-        self._wait()
-
-        ng_stats = []
-        for msg in self.rcv_msgs:
-            assert isinstance(
-                msg, ofproto_v1_3_parser.OFPMeterConfigStatsReply)
-            for stats in msg.body:
-                result, stats = self._compare_meter(stats, meter_mod)
-                if result:
-                    return
-                else:
-                    ng_stats.append(stats)
-        raise TestFailure(self.state, meters=', '.join(ng_stats))
+        error_dict = {
+            OpenFlowSw.send_flow_stats.__name__:
+                {'flows': ', '.join(ng_stats)},
+            OpenFlowSw.send_meter_config_stats.__name__:
+                {'meters': ', '.join(ng_stats)},
+            OpenFlowSw.send_group_desc_stats.__name__:
+                {'groups': ', '.join(ng_stats)}
+        }
+        raise TestFailure(self.state, **error_dict[method.__name__])
 
     def _test_get_packet_count(self, is_target):
         sw = self.target_sw if is_target else self.tester_sw
@@ -722,20 +726,24 @@ class OfTester(app_manager.RyuApp):
     def _continuous_packet_send(self, pkt):
         assert self.ingress_event is None
 
-        pkt_data = pkt[KEY_PACKETS][KEY_DATA]
+        pkt_text = pkt[KEY_PACKETS]['packet_text']
+        pkt_bin = pkt[KEY_PACKETS]['packet_binary']
         pktps = pkt[KEY_PACKETS][KEY_PKTPS]
         duration_time = pkt[KEY_PACKETS][KEY_DURATION_TIME]
+        randomize = pkt[KEY_PACKETS]['randomize']
 
-        self.logger.debug("send_packet:[%s]", packet.Packet(pkt_data))
+        self.logger.debug("send_packet:[%s]", packet.Packet(pkt_bin))
         self.logger.debug("pktps:[%d]", pktps)
         self.logger.debug("duration_time:[%d]", duration_time)
 
-        arg = {'pkt_data': pkt_data,
+        arg = {'packet_text': pkt_text,
+               'packet_binary': pkt_bin,
                'thread_counter': 0,
                'dot_span': int(CONTINUOUS_PROGRESS_SPAN /
                                CONTINUOUS_THREAD_INTVL),
                'packet_counter': float(0),
-               'packet_counter_inc': pktps * CONTINUOUS_THREAD_INTVL}
+               'packet_counter_inc': pktps * CONTINUOUS_THREAD_INTVL,
+               'randomize': randomize}
 
         try:
             self.ingress_event = hub.Event()
@@ -772,8 +780,14 @@ class OfTester(app_manager.RyuApp):
         self.ingress_threads.append(tid)
         hub.sleep(0)
         for _ in range(count):
+            if arg['randomize']:
+                msg = eval('/'.join(arg['packet_text']))
+                msg.serialize()
+                data = msg.data
+            else:
+                data = arg['packet_binary']
             try:
-                self.tester_sw.send_packet_out(arg['pkt_data'])
+                self.tester_sw.send_packet_out(data)
             except Exception as err:
                 self.thread_msg = err
                 self.ingress_event.set()
@@ -840,6 +854,18 @@ class OfTester(app_manager.RyuApp):
                 for attr in attr_list:
                     meter_stats.append('%s=%s' % (attr, getattr(stats1, attr)))
                 return False, 'meter_stats(%s)' % ','.join(meter_stats)
+        return True, None
+
+    def _compare_group(self, stats1, stats2):
+        attr_list = ['type', 'group_id', 'buckets']
+        for attr in attr_list:
+            value1 = getattr(stats1, attr)
+            value2 = getattr(stats2, attr)
+            if str(value1) != str(value2):
+                group_stats = []
+                for attr in attr_list:
+                    group_stats.append('%s=%s' % (attr, getattr(stats1, attr)))
+                return False, 'group_stats(%s)' % ','.join(group_stats)
             return True, None
 
     def _diff_packets(self, model_pkt, rcv_pkt):
@@ -966,45 +992,32 @@ class OfTester(app_manager.RyuApp):
                 self.rcv_msgs[0], ofproto_v1_3_parser.OFPErrorMsg)):
             raise TestReceiveError(self.state, self.rcv_msgs[0])
 
-    @set_ev_cls(ofp_event.EventOFPFlowStatsReply, handler.MAIN_DISPATCHER)
-    def flow_stats_reply_handler(self, ev):
-        state_list = [STATE_FLOW_EXIST_CHK,
-                      STATE_THROUGHPUT_FLOW_EXIST_CHK,
-                      STATE_GET_THROUGHPUT]
-        if self.state in state_list:
-            if self.waiter and ev.msg.xid in self.send_msg_xids:
-                self.rcv_msgs.append(ev.msg)
-                if not ev.msg.flags & ofproto_v1_3.OFPMPF_REPLY_MORE:
-                    self.waiter.set()
-                    hub.sleep(0)
-
-    @set_ev_cls(ofp_event.EventOFPMeterConfigStatsReply,
+    @set_ev_cls([ofp_event.EventOFPFlowStatsReply,
+                 ofp_event.EventOFPMeterConfigStatsReply,
+                 ofp_event.EventOFPTableStatsReply,
+                 ofp_event.EventOFPPortStatsReply,
+                 ofp_event.EventOFPGroupDescStatsReply],
                 handler.MAIN_DISPATCHER)
-    def meter_config_stats_reply_handler(self, ev):
-        state_list = [STATE_METER_EXIST_CHK]
-        if self.state in state_list:
-            if self.waiter and ev.msg.xid in self.send_msg_xids:
-                self.rcv_msgs.append(ev.msg)
-                if not ev.msg.flags & ofproto_v1_3.OFPMPF_REPLY_MORE:
-                    self.waiter.set()
-                    hub.sleep(0)
-
-    @set_ev_cls(ofp_event.EventOFPTableStatsReply, handler.MAIN_DISPATCHER)
-    def table_stats_reply_handler(self, ev):
-        state_list = [STATE_GET_MATCH_COUNT,
-                      STATE_FLOW_UNMATCH_CHK]
-        if self.state in state_list:
-            if self.waiter and ev.msg.xid in self.send_msg_xids:
-                self.rcv_msgs.append(ev.msg)
-                if not ev.msg.flags & ofproto_v1_3.OFPMPF_REPLY_MORE:
-                    self.waiter.set()
-                    hub.sleep(0)
-
-    @set_ev_cls(ofp_event.EventOFPPortStatsReply, handler.MAIN_DISPATCHER)
-    def port_stats_reply_handler(self, ev):
-        state_list = [STATE_TARGET_PKT_COUNT,
-                      STATE_TESTER_PKT_COUNT]
-        if self.state in state_list:
+    def stats_reply_handler(self, ev):
+        # keys: stats reply event classes
+        # values: states in which the events should be processed
+        event_states = {
+            ofp_event.EventOFPFlowStatsReply:
+                [STATE_FLOW_EXIST_CHK,
+                 STATE_THROUGHPUT_FLOW_EXIST_CHK,
+                 STATE_GET_THROUGHPUT],
+            ofp_event.EventOFPMeterConfigStatsReply:
+                [STATE_METER_EXIST_CHK],
+            ofp_event.EventOFPTableStatsReply:
+                [STATE_GET_MATCH_COUNT,
+                 STATE_FLOW_UNMATCH_CHK],
+            ofp_event.EventOFPPortStatsReply:
+                [STATE_TARGET_PKT_COUNT,
+                 STATE_TESTER_PKT_COUNT],
+            ofp_event.EventOFPGroupDescStatsReply:
+                [STATE_GROUP_EXIST_CHK]
+        }
+        if self.state in event_states[ev.__class__]:
             if self.waiter and ev.msg.xid in self.send_msg_xids:
                 self.rcv_msgs.append(ev.msg)
                 if not ev.msg.flags & ofproto_v1_3.OFPMPF_REPLY_MORE:
@@ -1016,9 +1029,11 @@ class OfTester(app_manager.RyuApp):
         state_list = [STATE_INIT_FLOW,
                       STATE_INIT_THROUGHPUT_FLOW,
                       STATE_INIT_METER,
+                      STATE_INIT_GROUP,
                       STATE_FLOW_INSTALL,
                       STATE_THROUGHPUT_FLOW_INSTALL,
                       STATE_METER_INSTALL,
+                      STATE_GROUP_INSTALL,
                       STATE_SEND_BARRIER]
         if self.state in state_list:
             if self.waiter and ev.msg.xid in self.send_msg_xids:
@@ -1052,7 +1067,7 @@ class OpenFlowSw(object):
         self.dp = dp
         self.logger = logger
 
-    def _send_msg(self, msg):
+    def send_msg(self, msg):
         if isinstance(self.dp, DummyDatapath):
             raise TestError(STATE_DISCONNECTED)
         msg.xid = None
@@ -1060,30 +1075,62 @@ class OpenFlowSw(object):
         self.dp.send_msg(msg)
         return msg.xid
 
-    def add_flow(self, flow_mod=None, in_port=None, out_port=None):
+    def add_flow(self, in_port=None, out_port=None):
         """ Add flow. """
         ofp = self.dp.ofproto
         parser = self.dp.ofproto_parser
 
-        if flow_mod:
-            mod = flow_mod
-        else:
-            match = parser.OFPMatch(in_port=in_port)
-            max_len = (0 if out_port != ofp.OFPP_CONTROLLER
-                       else ofp.OFPCML_MAX)
-            actions = [parser.OFPActionOutput(out_port, max_len)]
-            inst = [parser.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS,
-                                                 actions)]
-            mod = parser.OFPFlowMod(self.dp, cookie=0,
-                                    command=ofp.OFPFC_ADD,
-                                    match=match, instructions=inst)
-        return self._send_msg(mod)
+        match = parser.OFPMatch(in_port=in_port)
+        max_len = (0 if out_port != ofp.OFPP_CONTROLLER
+                   else ofp.OFPCML_MAX)
+        actions = [parser.OFPActionOutput(out_port, max_len)]
+        inst = [parser.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS,
+                                             actions)]
+        mod = parser.OFPFlowMod(self.dp, cookie=0,
+                                command=ofp.OFPFC_ADD,
+                                match=match, instructions=inst)
+        return self.send_msg(mod)
+
+    def del_flows(self, cookie=0):
+        """ Delete all flow except default flow. """
+        ofp = self.dp.ofproto
+        parser = self.dp.ofproto_parser
+        cookie_mask = 0
+        if cookie:
+            cookie_mask = 0xffffffffffffffff
+        mod = parser.OFPFlowMod(self.dp,
+                                cookie=cookie,
+                                cookie_mask=cookie_mask,
+                                table_id=ofp.OFPTT_ALL,
+                                command=ofp.OFPFC_DELETE,
+                                out_port=ofp.OFPP_ANY,
+                                out_group=ofp.OFPG_ANY)
+        return self.send_msg(mod)
+
+    def del_meters(self):
+        """ Delete all meter entries. """
+        ofp = self.dp.ofproto
+        parser = self.dp.ofproto_parser
+        mod = parser.OFPMeterMod(self.dp,
+                                 command=ofp.OFPMC_DELETE,
+                                 flags=0,
+                                 meter_id=ofp.OFPM_ALL)
+        return self.send_msg(mod)
+
+    def del_groups(self):
+        ofp = self.dp.ofproto
+        parser = self.dp.ofproto_parser
+        mod = parser.OFPGroupMod(self.dp,
+                                 command=ofp.OFPGC_DELETE,
+                                 type_=0,
+                                 group_id=ofp.OFPG_ALL)
+        return self.send_msg(mod)
 
     def send_barrier_request(self):
         """ send a BARRIER_REQUEST message."""
         parser = self.dp.ofproto_parser
         req = parser.OFPBarrierRequest(self.dp)
-        return self._send_msg(req)
+        return self.send_msg(req)
 
     def send_port_stats(self):
         """ Get port stats."""
@@ -1091,7 +1138,7 @@ class OpenFlowSw(object):
         parser = self.dp.ofproto_parser
         flags = 0
         req = parser.OFPPortStatsRequest(self.dp, flags, ofp.OFPP_ANY)
-        return self._send_msg(req)
+        return self.send_msg(req)
 
     def send_flow_stats(self):
         """ Get all flow. """
@@ -1100,63 +1147,24 @@ class OpenFlowSw(object):
         req = parser.OFPFlowStatsRequest(self.dp, 0, ofp.OFPTT_ALL,
                                          ofp.OFPP_ANY, ofp.OFPG_ANY,
                                          0, 0, parser.OFPMatch())
-        return self._send_msg(req)
-
-
-class TargetSw(OpenFlowSw):
-    def __init__(self, dp, logger):
-        super(TargetSw, self).__init__(dp, logger)
-
-    def del_test_flow(self):
-        """ Delete all flow except default flow. """
-        ofp = self.dp.ofproto
-        parser = self.dp.ofproto_parser
-        mod = parser.OFPFlowMod(self.dp,
-                                table_id=ofp.OFPTT_ALL,
-                                command=ofp.OFPFC_DELETE,
-                                out_port=ofp.OFPP_ANY,
-                                out_group=ofp.OFPG_ANY)
-        return self._send_msg(mod)
-
-    def del_test_meter(self):
-        """ Delete all meter entries. """
-        ofp = self.dp.ofproto
-        parser = self.dp.ofproto_parser
-        mod = parser.OFPMeterMod(self.dp,
-                                 command=ofp.OFPMC_DELETE,
-                                 flags=0,
-                                 meter_id=ofp.OFPM_ALL)
-        return self._send_msg(mod)
+        return self.send_msg(req)
 
     def send_meter_config_stats(self):
         """ Get all meter. """
         parser = self.dp.ofproto_parser
         stats = parser.OFPMeterConfigStatsRequest(self.dp)
-        return self._send_msg(stats)
+        return self.send_msg(stats)
+
+    def send_group_desc_stats(self):
+        parser = self.dp.ofproto_parser
+        stats = parser.OFPGroupDescStatsRequest(self.dp)
+        return self.send_msg(stats)
 
     def send_table_stats(self):
         """ Get table stats. """
         parser = self.dp.ofproto_parser
         req = parser.OFPTableStatsRequest(self.dp, 0)
-        return self._send_msg(req)
-
-
-class TesterSw(OpenFlowSw):
-    def __init__(self, dp, logger):
-        super(TesterSw, self).__init__(dp, logger)
-
-    def del_flows_for_throughput_analysis(self):
-        """ Delete all flow except default flow. """
-        ofp = self.dp.ofproto
-        parser = self.dp.ofproto_parser
-        mod = parser.OFPFlowMod(self.dp,
-                                cookie=THROUGHPUT_COOKIE,
-                                cookie_mask=0xffffffffffffffff,
-                                table_id=ofp.OFPTT_ALL,
-                                command=ofp.OFPFC_DELETE,
-                                out_port=ofp.OFPP_ANY,
-                                out_group=ofp.OFPG_ANY)
-        return self._send_msg(mod)
+        return self.send_msg(req)
 
     def send_packet_out(self, data):
         """ send a PacketOut message."""
@@ -1166,7 +1174,7 @@ class TesterSw(OpenFlowSw):
         out = parser.OFPPacketOut(
             datapath=self.dp, buffer_id=ofp.OFP_NO_BUFFER,
             data=data, in_port=ofp.OFPP_CONTROLLER, actions=actions)
-        return self._send_msg(out)
+        return self.send_msg(out)
 
 
 class TestPatterns(dict):
@@ -1239,9 +1247,9 @@ class Test(stringify.StringifyMixin):
 
         # parse 'prerequisite'
         prerequisite = []
-        if not KEY_PREREQ in buf:
+        if KEY_PREREQ not in buf:
             raise ValueError('a test requires a "%s" block' % KEY_PREREQ)
-        allowed_mod = [KEY_FLOW, KEY_METER]
+        allowed_mod = [KEY_FLOW, KEY_METER, KEY_GROUP]
         for flow in buf[KEY_PREREQ]:
             key, value = flow.popitem()
             if key not in allowed_mod:
@@ -1257,7 +1265,7 @@ class Test(stringify.StringifyMixin):
 
         # parse 'tests'
         tests = []
-        if not KEY_TESTS in buf:
+        if KEY_TESTS not in buf:
             raise ValueError('a test requires a "%s" block.' % KEY_TESTS)
 
         for test in buf[KEY_TESTS]:
@@ -1268,18 +1276,22 @@ class Test(stringify.StringifyMixin):
                                          KEY_PKT_IN, KEY_TBL_MISS))
             test_pkt = {}
             # parse 'ingress'
-            if not KEY_INGRESS in test:
+            if KEY_INGRESS not in test:
                 raise ValueError('a test requires "%s" field.' % KEY_INGRESS)
             if isinstance(test[KEY_INGRESS], list):
                 test_pkt[KEY_INGRESS] = __test_pkt_from_json(test[KEY_INGRESS])
             elif isinstance(test[KEY_INGRESS], dict):
                 test_pkt[KEY_PACKETS] = {
-                    KEY_DATA: __test_pkt_from_json(
+                    'packet_text': test[KEY_INGRESS][KEY_PACKETS][KEY_DATA],
+                    'packet_binary': __test_pkt_from_json(
                         test[KEY_INGRESS][KEY_PACKETS][KEY_DATA]),
                     KEY_DURATION_TIME: test[KEY_INGRESS][KEY_PACKETS].get(
                         KEY_DURATION_TIME, DEFAULT_DURATION_TIME),
                     KEY_PKTPS: test[KEY_INGRESS][KEY_PACKETS].get(
-                        KEY_PKTPS, DEFAULT_PKTPS)}
+                        KEY_PKTPS, DEFAULT_PKTPS),
+                    'randomize': True in [
+                        line.find('randint') != -1
+                        for line in test[KEY_INGRESS][KEY_PACKETS][KEY_DATA]]}
             else:
                 raise ValueError('invalid format: "%s" field' % KEY_INGRESS)
             # parse 'egress' or 'PACKET_IN' or 'table-miss'
@@ -1322,3 +1334,9 @@ class DummyDatapath(object):
     def __init__(self):
         self.ofproto = ofproto_v1_3
         self.ofproto_parser = ofproto_v1_3_parser
+
+    def set_xid(self, _):
+        pass
+
+    def send_msg(self, _):
+        pass
