@@ -1,5 +1,5 @@
-# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
-# Copyright (C) 2013 YAMAMOTO Takashi <yamamoto at valinux co jp>
+# Copyright (C) 2013,2014 Nippon Telegraph and Telephone Corporation.
+# Copyright (C) 2013,2014 YAMAMOTO Takashi <yamamoto at valinux co jp>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -47,6 +47,7 @@ class Test_bgp(unittest.TestCase):
                      bgp.BGPOptParamCapabilityRouteRefresh(),
                      bgp.BGPOptParamCapabilityMultiprotocol(
                          afi=afi.IP, safi=safi.MPLS_VPN),
+                     bgp.BGPOptParamCapabilityCarryingLabelInfo(),
                      bgp.BGPOptParamCapabilityFourOctetAsNumber(
                          as_number=1234567),
                      bgp.BGPOptParamUnknown(type_=99, value='fuga')]
@@ -78,27 +79,25 @@ class Test_bgp(unittest.TestCase):
                             bgp.BGPWithdrawnRoute(length=32,
                                                   addr='192.0.2.13')]
         mp_nlri = [
-            bgp._BinAddrPrefix(32, 'efgh\0\0'),
-            bgp._BinAddrPrefix(16, 'ij\0\0\0\0'),
+            bgp.LabelledVPNIPAddrPrefix(24, '192.0.9.0',
+                                        route_dist='100:100',
+                                        labels=[1, 2, 3]),
+            bgp.LabelledVPNIPAddrPrefix(26, '192.0.10.192',
+                                        route_dist='10.0.0.1:10000',
+                                        labels=[5, 6, 7, 8]),
         ]
         communities = [
             bgp.BGP_COMMUNITY_NO_EXPORT,
             bgp.BGP_COMMUNITY_NO_ADVERTISE,
         ]
         ecommunities = [
-            bgp.BGPTwoOctetAsSpecificExtendedCommunity(subtype=1,
-                                                       as_number=65500,
-                                                       local_administrator=
-                                                       3908876543),
-            bgp.BGPFourOctetAsSpecificExtendedCommunity(subtype=2,
-                                                        as_number=10000000,
-                                                        local_administrator=
-                                                        59876),
-            bgp.BGPIPv4AddressSpecificExtendedCommunity(subtype=3,
-                                                        ipv4_address=
-                                                        '192.0.2.1',
-                                                        local_administrator=
-                                                        65432),
+            bgp.BGPTwoOctetAsSpecificExtendedCommunity(
+                subtype=1, as_number=65500, local_administrator=3908876543),
+            bgp.BGPFourOctetAsSpecificExtendedCommunity(
+                subtype=2, as_number=10000000, local_administrator=59876),
+            bgp.BGPIPv4AddressSpecificExtendedCommunity(
+                subtype=3, ipv4_address='192.0.2.1',
+                local_administrator=65432),
             bgp.BGPOpaqueExtendedCommunity(opaque='abcdefg'),
             bgp.BGPUnknownExtendedCommunity(type_=99, value='abcdefg'),
         ]
@@ -123,7 +122,7 @@ class Test_bgp(unittest.TestCase):
                                             nlri=mp_nlri),
             bgp.BGPPathAttributeMpUnreachNLRI(afi=afi.IP, safi=safi.MPLS_VPN,
                                               withdrawn_routes=mp_nlri),
-            bgp.BGPPathAttributeUnknown(flags=0, type_=100, value=300*'bar')
+            bgp.BGPPathAttributeUnknown(flags=0, type_=100, value=300 * 'bar')
         ]
         nlri = [
             bgp.BGPNLRI(length=24, addr='203.0.113.1'),
@@ -223,27 +222,25 @@ class Test_bgp(unittest.TestCase):
                             bgp.BGPWithdrawnRoute(length=32,
                                                   addr='192.0.2.13')]
         mp_nlri = [
-            bgp._BinAddrPrefix(32, 'efgh\0\0'),
-            bgp._BinAddrPrefix(16, 'ij\0\0\0\0'),
+            bgp.LabelledVPNIPAddrPrefix(24, '192.0.9.0',
+                                        route_dist='100:100',
+                                        labels=[1, 2, 3]),
+            bgp.LabelledVPNIPAddrPrefix(26, '192.0.10.192',
+                                        route_dist='10.0.0.1:10000',
+                                        labels=[5, 6, 7, 8]),
         ]
         communities = [
             bgp.BGP_COMMUNITY_NO_EXPORT,
             bgp.BGP_COMMUNITY_NO_ADVERTISE,
         ]
         ecommunities = [
-            bgp.BGPTwoOctetAsSpecificExtendedCommunity(subtype=1,
-                                                       as_number=65500,
-                                                       local_administrator=
-                                                       3908876543),
-            bgp.BGPFourOctetAsSpecificExtendedCommunity(subtype=2,
-                                                        as_number=10000000,
-                                                        local_administrator=
-                                                        59876),
-            bgp.BGPIPv4AddressSpecificExtendedCommunity(subtype=3,
-                                                        ipv4_address=
-                                                        '192.0.2.1',
-                                                        local_administrator=
-                                                        65432),
+            bgp.BGPTwoOctetAsSpecificExtendedCommunity(
+                subtype=1, as_number=65500, local_administrator=3908876543),
+            bgp.BGPFourOctetAsSpecificExtendedCommunity(
+                subtype=2, as_number=10000000, local_administrator=59876),
+            bgp.BGPIPv4AddressSpecificExtendedCommunity(
+                subtype=3, ipv4_address='192.0.2.1',
+                local_administrator=65432),
             bgp.BGPOpaqueExtendedCommunity(opaque='abcdefg'),
             bgp.BGPUnknownExtendedCommunity(type_=99, value='abcdefg'),
         ]
@@ -268,7 +265,7 @@ class Test_bgp(unittest.TestCase):
                                             nlri=mp_nlri),
             bgp.BGPPathAttributeMpUnreachNLRI(afi=afi.IP, safi=safi.MPLS_VPN,
                                               withdrawn_routes=mp_nlri),
-            bgp.BGPPathAttributeUnknown(flags=0, type_=100, value=300*'bar')
+            bgp.BGPPathAttributeUnknown(flags=0, type_=100, value=300 * 'bar')
         ]
         nlri = [
             bgp.BGPNLRI(length=24, addr='203.0.113.1'),
